from .base import ZKBase


class ClientTest(ZKBase):

    async def test_childs(self):
        await self.c.create('/test')
        await self.c.create('/test/subtest')
        await self.c.create('/test/subtest2')
        c = await self.c.get_children('/test')
        assert len(c) == 2, c
        self.assertSetEqual({'subtest', 'subtest2'}, set(c))
        await self.c.delete('/test/subtest2')
        await self.c.delete('/test/subtest')
        await self.c.delete('/test')



import asyncio

from .base import ZKBase


NAME = '/test_barrier'


class TestBarrier(ZKBase):
    async def worker(self):
        barrier = self.c.recipes.Barrier(NAME)
        await barrier.wait()
        while True:
            self.assertEqual(self.lifted, True)
            await asyncio.sleep(0.3)

    async def test_barrier(self):
        self.lifted = False
        barrier = self.c.recipes.Barrier(NAME)
        await barrier.create()
        asyncio.ensure_future(self.worker())
        await asyncio.sleep(0.05)
        await barrier.lift()
        self.lifted = True
        await asyncio.sleep(0.1)


class TestDoubleBarrier(ZKBase):
    async def worker(self, num, min_workers):
        c = await self.client()
        barrier = c.recipes.DoubleBarrier(NAME, min_workers)

        await asyncio.sleep(0.01*num)
        await barrier.enter()
        for i in range(5):
            self.assertGreaterEqual(self.num_workers, min_workers)
            await asyncio.sleep(0.01*num)
        await barrier.leave()

    async def test_barrier(self):
        self.lifted = False
        self.num_workers = 0
        workers = []
        target = 8
        for x in range(target):
            self.num_workers += 1
            workers.append(asyncio.ensure_future(self.worker(x, target)))
        await asyncio.wait(workers)

    async def tearDown(self):
        await self.c.delete(NAME)
        await super().tearDown()



import os
from aiozk import ZKClient, exc
from .aio_test import AIOTestCase


HOST = os.environ.get('ZK_HOST', 'zk')


class ZKBase(AIOTestCase):

    async def client(self):
        c = ZKClient(HOST, chroot='/test_aiozk')
        await c.start()
        self.clients.append(c)
        return c

    async def setUp(self):
        self.clients = []
        self.c = ZKClient(HOST, chroot='/test_aiozk')
        await self.c.start()
        if len(await self.c.get_children('/')) > 0:
            await self.c.deleteall('')
            await self.c.create('')

    async def tearDown(self):
        for c in self.clients:
            await c.close()
        try:
            await self.c.delete('/')
        except exc.NotEmpty:
            await self.dump_tree()
            await self.c.deleteall('')
        await self.c.close()

    async def dump_tree(self, base='/'):
        out = list(await self.get_tree(base))
        print('Tree dump: {}'.format(out))
        return out

    async def get_tree(self, curr='/'):
        out = [curr]
        childs = await self.c.get_children(curr)
        for c in childs:
            # eliminate double slash: //root = '/'.join('/', 'root')
            if curr == '/':
                curr = ''
            out.append(await self.get_tree('/'.join([curr, c])))
        return out
